@@ -13,6 +13,7 @@ use Test::Builder;
 	file_min_size_ok file_readable_ok file_not_readable_ok file_writeable_ok
 	file_not_writeable_ok file_executable_ok file_not_executable_ok
 	file_mode_is file_mode_isnt
+	file_mode_has file_mode_hasnt
 	file_is_symlink_ok
 	symlink_target_exists_ok symlink_target_is
 	symlink_target_dangles_ok
@@ -22,9 +23,12 @@ use Test::Builder;
 	group_is group_isnt
 	file_line_count_is file_line_count_isnt file_line_count_between
 	file_contains_like file_contains_unlike
+	file_contains_utf8_like file_contains_utf8_unlike
+	file_contains_encoded_like file_contains_encoded_unlike
+	file_mtime_gt_ok file_mtime_lt_ok file_mtime_age_ok
 	);
 
-$VERSION = '1.35_01';
+$VERSION = '1.43_02';
 
 {
 use warnings;
@@ -32,13 +36,15 @@ use warnings;
 
 my $Test = Test::Builder->new();
 
+=encoding utf8
+
 =head1 NAME
 
 Test::File -- test file attributes
 
 =head1 SYNOPSIS
 
-use Test::File;
+  use Test::File;
 
 =head1 DESCRIPTION
 
@@ -53,10 +59,13 @@ matter the permissions.
 Some attributes don't make sense outside of Unix, either, so some
 tests automatically skip if they think they won't work on the
 platform.  If you have a way to make these functions work on Windows,
-for instance, please send me a patch. :)
+for instance, please send me a patch. :) IF you want to pretend to be
+Windows on a non-Windows machine (for instance, to test C<skip()>),
+you can set the C<PRETEND_TO_BE_WINDOWS> environment variable.
 
-The optional NAME parameter for every function allows you to specify a name for the test.  If not
-supplied, a reasonable default will be generated.
+The optional NAME parameter for every function allows you to specify a
+name for the test.  If not supplied, a reasonable default will be
+generated.
 
 =head2 Functions
 
@@ -75,6 +84,7 @@ sub _normalize
 sub _win32
 	{
 	return 0 if $^O eq 'darwin';
+	return $ENV{PRETEND_TO_BE_WIN32} if defined $ENV{PRETEND_TO_BE_WIN32};
 	return $^O =~ m/Win/;
 	}
 
@@ -563,24 +573,67 @@ function as well.
 
 Contributed by Buddy Burden C<< <barefoot@cpan.org> >>.
 
+=item file_contains_utf8_like ( FILENAME, PATTERN [, NAME ] )
+
+The same as C<file_contains_like>, except the file is opened as UTF-8.
+
+=item file_contains_utf8_unlike ( FILENAME, PATTERN [, NAME ] )
+
+The same as C<file_contains_unlike>, except the file is opened as UTF-8.
+
+=item file_contains_encoded_like ( FILENAME, ENCODING, PATTERN [, NAME ] )
+
+The same as C<file_contains_like>, except the file is opened with ENCODING
+
+=item file_contains_encoded_unlike ( FILENAME, ENCODING, PATTERN [, NAME ] )
+
+The same as C<file_contains_unlike>, except the file is opened with ENCODING.
+
 =cut
 
 sub file_contains_like
 	{
 		local $Test::Builder::Level = $Test::Builder::Level + 1;
-		_file_contains(like => "contains", @_);
+		_file_contains(like => "contains", undef, @_);
 	}
 
 sub file_contains_unlike
 	{
 		local $Test::Builder::Level = $Test::Builder::Level + 1;
-		_file_contains(unlike => "doesn't contain", @_);
+		_file_contains(unlike => "doesn't contain", undef, @_);
+	}
+
+sub file_contains_utf8_like
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		_file_contains(like => "contains", 'UTF-8', @_);
+	}
+sub file_contains_utf8_unlike
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		_file_contains(unlike => "doesn't contain", 'UTF-8', @_);
+	}
+
+sub file_contains_encoded_like
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		my $filename = shift;
+		my $encoding = shift;
+		_file_contains(like => "contains", $encoding, $filename, @_);
+	}
+sub file_contains_encoded_unlike
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		my $filename = shift;
+		my $encoding = shift;
+		_file_contains(unlike => "doesn't contain", $encoding, $filename, @_);
 	}
 
 sub _file_contains
 	{
 	my $method   = shift;
 	my $verb     = shift;
+	my $encoding = shift;
 	my $filename = _normalize( shift );
 	my $patterns = shift;
 	my $name     = shift;
@@ -621,6 +674,11 @@ sub _file_contains
 		$Test->diag( "Could not open [$filename]: \$! is [$!]!" );
 		return $Test->ok( 0, $name );
 		}
+
+	if (defined $encoding) {
+		binmode FH, ":encoding($encoding)";
+	}
+
 	local $/ = undef;
 	$file_contents = <FH>;
 	close FH;
@@ -876,6 +934,90 @@ sub file_mode_isnt
 		}
 	}
 
+=item file_mode_has( FILENAME, MODE [, NAME ] )
+
+Ok if the file exists and has all the bits in mode turned on, not ok
+if the file does not exist or the mode does not match.  That is,
+C<< FILEMODE & MODE == MODE >> must be true.
+
+This test automatically skips if it thinks it is on a
+Windows platform.
+
+Contributed by Ricardo Signes C<< <rjbs@cpan.org> >>
+
+=cut
+
+sub file_mode_has
+	{
+	if( _win32() )
+		{
+		$Test->skip( "file_mode_has doesn't work on Windows!" );
+		return;
+		}
+
+	my $filename = _normalize( shift );
+	my $mode     = shift;
+
+	my $name     = shift || sprintf("%s mode has all bits of %04o", $filename, $mode);
+
+	my $present = -e $filename;
+	my $gotmode = $present ? (stat($filename))[2] : undef;
+	my $ok      = $present && ($gotmode & $mode) == $mode;
+
+	if( $ok )
+		{
+		$Test->ok(1, $name);
+		}
+	else
+		{
+		my $missing = ($gotmode ^ $mode) & $mode;
+		$Test->diag(sprintf("File [%s] mode is missing component %04o!", $filename, $missing) );
+		$Test->ok(0, $name);
+		}
+	}
+
+=item file_mode_hasnt( FILENAME, MODE [, NAME ] )
+
+Ok if the file exists and has all the bits in mode turned off, not ok
+if the file does not exist or the mode does not match.  That is,
+C<< FILEMODE & MODE == 0 >> must be true.
+
+This test automatically skips if it thinks it is on a
+Windows platform.
+
+Contributed by Ricardo Signes C<< <rjbs@cpan.org> >>
+
+=cut
+
+sub file_mode_hasnt
+	{
+	if( _win32() )
+		{
+		$Test->skip( "file_mode_hasnt doesn't work on Windows!" );
+		return;
+		}
+
+	my $filename = _normalize( shift );
+	my $mode     = shift;
+
+	my $name     = shift || sprintf("%s mode has no bits of %04o", $filename, $mode);
+
+	my $present = -e $filename;
+	my $gotmode = $present ? (stat($filename))[2] : undef;
+	my $ok      = $present && ($gotmode & $mode) == 0;
+
+	if( $ok )
+		{
+		$Test->ok(1, $name);
+		}
+	else
+		{
+		my $bad = $gotmode & $mode;
+		$Test->diag(sprintf("File [%s] mode has forbidden component %04o!", $filename, $bad) );
+		$Test->ok(0, $name);
+		}
+	}
+
 =item file_is_symlink_ok( FILENAME [, NAME ] )
 
 Ok if FILENAME is a symlink, even if it points to a non-existent
@@ -1050,8 +1192,6 @@ symlinks. If the file does not exist, the test fails.
 
 =cut
 
-=pod
-
 sub symlink_target_is_absolute_ok 
 	{
 	if( _no_symlinks_here() )
@@ -1061,30 +1201,29 @@ sub symlink_target_is_absolute_ok
 		return;
 		}
 
-	my $file = shift;
-	my $name = shift || "symlink $file points to an absolute path";
+	my( $from, $from_base, $to, $to_base, $name ) = @_;
+	my $link     = readlink( $from );
+	my $link_err = defined( $link ) ? '' : $!; # $! doesn't always get reset
+	my $link_abs = abs_path( rel2abs($link, $from_base) );
+	my $to_abs   = abs_path( rel2abs($to, $to_base) );
 
-my ($from, $from_base, $to, $to_base, $name) = @_;
-my $link   = readlink( $from );
-my $link_err = defined( $link ) ? '' : $!; # $! doesn't always get reset
-my $link_abs = abs_path( rel2abs($link, $from_base) );
-my $to_abs  = abs_path( rel2abs($to, $to_base) );
+	if (defined( $link_abs ) && defined( $to_abs ) && $link_abs eq $to_abs) {
+		$Test->ok( 1, $name );
+		}
+	else {
+		$Test->ok( 0, $name );
+		$link     ||= 'undefined';
+		$link_abs ||= 'undefined';
+		$to_abs   ||= 'undefined';
 
-if (defined( $link_abs ) && defined( $to_abs ) && $link_abs eq $to_abs) {
- $Test->ok( 1, $name );
-} else {
- $Test->ok( 0, $name );
- $link   ||= 'undefined';
- $link_abs ||= 'undefined';
- $to_abs  ||= 'undefined';
- $Test->diag("    link: $from");
- $Test->diag("     got: $link");
- $Test->diag("    (abs): $link_abs");
- $Test->diag("  expected: $to");
- $Test->diag("    (abs): $to_abs");
- $Test->diag("  readlink() error: $link_err") if ($link_err);
-}
-}
+		$Test->diag("    link: $from");
+		$Test->diag("     got: $link");
+		$Test->diag("    (abs): $link_abs");
+		$Test->diag("  expected: $to");
+		$Test->diag("    (abs): $to_abs");
+		$Test->diag("  readlink() error: $link_err") if ($link_err);
+		}
+	}
 
 =item dir_exists_ok( DIRECTORYNAME [, NAME ] )
 
@@ -1367,7 +1506,7 @@ Contributed by Dylan Martin
 =cut
 
 sub group_is
- 	{
+	{
 	my $filename      = shift;
 	my $group         = shift;
 	my $name          = ( shift || "$filename belongs to group $group" );
@@ -1386,7 +1525,7 @@ sub group_is
 	my $file_gid  = ( stat $filename )[5];
 
 	unless( defined $file_gid )
- 		{
+		{
 		$Test->skip("stat failed to return group gid for $filename!");
 		return;
 		}
@@ -1397,8 +1536,8 @@ sub group_is
 	unless( defined $real_group )
 		{
 		$Test->diag("File does not belong to $group!");
- 		return $Test->ok( 0, $name );
- 		}
+		return $Test->ok( 0, $name );
+		}
 
 	$Test->diag( "File [$filename] belongs to $real_group ($file_gid), ".
 			"not $group ($group_gid)!" );
@@ -1439,7 +1578,7 @@ sub group_isnt
 	return $Test->ok( 1, $name ) if $file_gid != $group_gid;
 
 	$Test->diag( "File [$filename] belongs to $group ($group_gid)!" );
- 		return $Test->ok( 0, $name );
+		return $Test->ok( 0, $name );
 	}
 
 sub _get_uid
@@ -1478,6 +1617,119 @@ sub _get_gid
 	$group_uid;
 	}
 
+
+=item file_mtime_age_ok( FILE [, WITHIN_SECONDS ] [, NAME ] )
+
+Ok if FILE's modified time is WITHIN_SECONDS inclusive of the system's current time.
+This test uses stat() to obtain the mtime. If the file does not exist the test
+returns failure. If stat() fails, the test is skipped.
+
+=cut
+
+sub file_mtime_age_ok
+	{
+	my $filename    = shift;
+	my $within_secs = int shift || 0;
+	my $name        = shift || "$filename mtime within $within_secs seconds of current time";
+
+	my $time        = time();
+
+	my $filetime = _stat_file($filename, 9);
+
+	return if ( $filetime == -1 ); #skip
+
+	return $Test->ok(1, $name) if ( $filetime + $within_secs > $time-1  );
+
+	$Test->diag( "Filename [$filename] mtime [$filetime] is not $within_secs seconds within current system time [$time].");
+	return $Test->ok(0, $name);
+	}
+
+=item file_mtime_gt_ok( FILE, UNIXTIME [, NAME ] )
+
+Ok if FILE's mtime is > UNIXTIME. This test uses stat() to get the mtime. If stat() fails
+this test is skipped. If FILE does not exist, this test fails.
+
+=cut
+
+sub file_mtime_gt_ok
+	{
+	my $filename    = shift;
+	my $time        = int shift;
+	my $name        = shift || "$filename mtime is less than unix timestamp $time";
+
+	my $filetime = _stat_file($filename, 9);
+
+	return if ( $filetime == -1 ); #skip
+
+	return $Test->ok(1, $name) if ( $filetime > $time );
+
+	$Test->diag( "Filename [$filename] mtime [$filetime] not greater than $time" );
+	$Test->ok(0, $name);
+  }
+
+=item file_mtime_lt_ok( FILE, UNIXTIME, [, NAME ] )
+
+Ok if FILE's modified time is < UNIXTIME.  This test uses stat() to get the mtime. If stat() fails
+this test is skipped. If FILE does not exist, this test fails.
+
+=cut
+
+sub file_mtime_lt_ok
+	{
+	my $filename = shift;
+	my $time = int shift;
+	my $name = shift || "$filename mtime less than unix timestamp $time";
+
+  # gets mtime
+	my $filetime = _stat_file($filename, 9);
+
+	return if ( $filetime == -1 ); #skip
+
+	return $Test->ok(1, $name) if ( $filetime < $time );
+
+	$Test->diag( "Filename [$filename] mtime [$filetime] not less than $time" );
+	$Test->ok(0, $name);
+	}
+
+# private function to safely stat a file
+#
+# Arugments:
+# filename     file to perform on
+# attr_pos     pos of the array returned from stat we want to compare. perldoc -f stat
+#
+# Returns:
+# -1        - stat failed
+#  0        - failure (file doesn't exist etc)
+#  filetime - on success, time requested provided by stat
+#
+sub _stat_file
+	{
+	my $filename    = _normalize( shift );
+	my $attr_pos    = shift;
+
+	unless( defined $filename )
+		{
+		$Test->diag( "Filename not specified!" );
+		return 0;
+		}
+
+	unless( -e $filename )
+		{
+		$Test->diag( "Filename [$filename] does not exist!" );
+		return 0;
+		}
+
+	my $filetime = ( stat($filename) )[$attr_pos];
+
+	unless( $filetime )
+		{
+		$Test->diag( "stat of $filename failed" );
+		return -1; #skip on stat failure
+		}
+
+	return $filetime;
+	}
+
 =back
 
 =head1 TO DO
@@ -1520,9 +1772,12 @@ Buddy Burden C<< <barefoot@cpan.org> >> provided C<dir_exists_ok>,
 C<dir_contains_ok>, C<file_contains_like>, and
 C<file_contains_unlike>.
 
+xmikew C<< <https://github.com/xmikew> >> provided the C<mtime_age>
+stuff.
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002-2013 brian d foy.  All rights reserved.
+Copyright © 2002-2015, brian d foy <bdfoy@cpan.org>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
